//	GamePlayState.cpp - The main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "FishGlobalData.h"
#include "GamePlayState.h"
#include "ColourHelpers.h"
#include "Random.h"
#include "Fish.h"

#include <sstream>

//------------------------------------------------------------------------------
GameStateCreator<GamePlayState> GamePlayState::creator;

//------------------------------------------------------------------------------
GamePlayState::GamePlayState(Application& app, SounderPtr& sound):
GameState(app, sound),
winWait(0.0f),
waitTime(false),
fadeOut(0.0f),
fadeOutTween(fadeOut, fadeOut, 0.02f),
sounder(sound)
{
	int numPlayers;

	fadeOutTween.setIndex(1.0f);

	flowers.push_back(TwoFloats(1068, 70));
	flowers.push_back(TwoFloats(539, 79));
	flowers.push_back(TwoFloats(145, 137));
	flowers.push_back(TwoFloats(791, 176));
	flowers.push_back(TwoFloats(1140, 285));
	flowers.push_back(TwoFloats(383, 286));
	flowers.push_back(TwoFloats(73, 390));
	flowers.push_back(TwoFloats(961, 420));
	flowers.push_back(TwoFloats(652, 433));
	flowers.push_back(TwoFloats(254, 506));
	flowers.push_back(TwoFloats(475, 553));
	flowers.push_back(TwoFloats(1213, 576));
	flowers.push_back(TwoFloats(800, 586));

	numPlayers = dynamic_pointer_cast<FishGlobalData>(getGlobalData())->getNumPlayers();
	if(numPlayers > 1)
	{
		int i, j;
		int index;
		int index2;
		ThreeFloats col;
		bool unavailable;
		float hue = 0.0f;
		std::vector<int> availableFlowers;

		std::vector<std::wstring> names;

		names.push_back(L"Fishy Bill");
		names.push_back(L"Tina Tuna");
		names.push_back(L"Simon Salmon");
		names.push_back(L"Moany Shark");
		names.push_back(L"Captain Haddock");
		names.push_back(L"Flounder Phil");
		names.push_back(L"Sad Clownfish");
		names.push_back(L"Calvin Caviar");
		names.push_back(L"Silly Seahorse");
		names.push_back(L"Cod & Chips");

		for(i=0;i<numPlayers;++i)
		{
			/*unavailable = false;
			for(j=0;j<fish.size();++j)
			{
				if(fish[j]->getDestFlower() == i)
				{
					unavailable = true;
					break;
				}
			}
			if(!unavailable)*/
				availableFlowers.push_back(i);
		}

		for(i=0;i<numPlayers;++i)
		{
			col.x = hue;
			col.y = 0.5f;
			col.z = 0.5f;
			col = ColourHelpers::hslToRgb(col);

			index = Random::getGlobal().getRandomInt(availableFlowers.size());
			index2 = Random::getGlobal().getRandomInt(names.size());
			fish.push_back(shared_ptr<Fish>(new Fish(flowers[availableFlowers[index]],
														col,
														names[i],
														availableFlowers[index])));

			hue += 1.0f/(float)numPlayers;
			availableFlowers.erase(availableFlowers.begin()+index);
			//names.erase(names.begin()+index2);
		}

		winner = L"";
		waitTime = true;
	}
}

//------------------------------------------------------------------------------
GamePlayState::~GamePlayState()
{

}

//------------------------------------------------------------------------------
void GamePlayState::update(float delta)
{
	int i, j;
	int lastFish = -1;
	TwoFloats tempPos;
	std::wstring winnerName;

	winParticles.update(delta);

	if(winWait > 0.0f)
		winWait -= delta;

	if(!fadeOutTween.getFinished())
	{
		fadeOutTween.tick(delta);
		if(fadeOutTween.getFinished())
		{
			nextState(L"TitleScreen");
			return;
		}
	}

	if(waitTime || (winner != L""))
	{
		for(i=0;i<fish.size();++i)
			fish[i]->initialUpdate(delta);

		return;
	}

	/*for(i=0;i<fish.size();++i)
	{
		for(j=i+1;j<fish.size();++j)
		{
			while((fish[i]->getDestKey() == fish[j]->getDestKey()) &&
				  (fish[i]->getDestKey() != L""))
			{
				fish[i]->updateDestKey();
			}
		}
	}*/

	for(i=(fish.size()-1);i>=0;--i)
	{
		fish[i]->update(delta);
		if(fish[i]->getPosition().y > 752.0f)
		{
			winnerName = fish[i]->getName();
			if(fish.size() > 1)
				fish.erase(fish.begin()+i);
			lastFish = i;
		}
	}

	if((fish.size() < 2) && (lastFish > -1))
	{
		if(fish.size() > 0)
		{
			winner = fish[0]->getName();
			fish[0]->setWinner();
		}
		else
		{
			winner = winnerName;
			fish[lastFish]->setWinner();
		}
		tempPos = getDrawingAreaSize() * 0.5f;
		tempPos.y -= 60.0f;
		winParticles.trigger(tempPos);

		winWait = 30.0f;
	}
}

//------------------------------------------------------------------------------
void GamePlayState::draw(DrawerPtr& d)
{
	int i;

	d->drawImage(L"Background", 0.0f);

	winParticles.draw(d);

	for(i=0;i<fish.size();++i)
		fish[i]->draw(d);

	if(winner != L"")
	{
		std::wstringstream tempstr;

		tempstr << winner << L" is the winner!";
		d->drawText(tempstr.str(),
					L"Font1",
					winnerLayout,
					TwoFloats(0.0f, 400.0f),
					d->getDrawingAreaSize().x,
					TextLayout::Centre);
		d->drawText(L"-space- for rematch  | -esc- for title",
					L"Font1",
					winnerLayout2,
					TwoFloats(0.0f, 690.0f),
					d->getDrawingAreaSize().x,
					TextLayout::Centre,
					1.0f,
					ThreeFloats(1.0f));
	}
	else if(waitTime)
	{
		d->drawText(L"Any key to start",
					L"Font1",
					TwoFloats(0.0f, 400.0f),
					d->getDrawingAreaSize().x,
					TextLayout::Centre);
	}

	if(!fadeOutTween.getFinished())
	{
		d->fillRect(0.0f,
					d->getDrawingAreaSize(),
					ThreeFloats(141.0f/255.0f,
								220.0f/255.0f,
								127.0f/255.0f),
					fadeOut);
	}
}

//------------------------------------------------------------------------------
void GamePlayState::keyDown(const std::wstring& action,
							const std::wstring& keyPressed,
							int keyModifier)
{
	int i, j;
	int numPlayers = -1;
	bool unavailable = false;
	std::vector<int> availableFlowers;

	if(keyPressed == L"F10")
		return;

	if(action == L"Escape")
		fadeOutTween.reset(1.0f);
		//nextState(L"TitleScreen");
	else if((winner != L"") && (winWait <= 0.0f))
	{
		if(action == L"Action")
			nextState(L"GamePlayState");
		else
			nextState(L"TitleScreen");
	}
	else if(waitTime)
		waitTime = false;
	/*else if((fish.size() == 0) || (winner != L""))
	{
		if(keyPressed == L"2")
			numPlayers = 2;
		else if(keyPressed == L"3")
			numPlayers = 3;
		else if(keyPressed == L"4")
			numPlayers = 4;
		else if(keyPressed == L"5")
			numPlayers = 5;
		else if(keyPressed == L"6")
			numPlayers = 6;
		else if(keyPressed == L"7")
			numPlayers = 7;
		else if(keyPressed == L"8")
			numPlayers = 8;
		else if(keyPressed == L"9")
			numPlayers = 9;

		if(numPlayers > -1)
		{
			ThreeFloats col;
			float hue = 0.0f;
			int index;
			int index2;

			std::vector<std::wstring> names;

			names.push_back(L"Fishy Bill");
			names.push_back(L"Tina Tuna");
			names.push_back(L"Simon Salmon");
			names.push_back(L"Moany Shark");
			names.push_back(L"Captain Haddock");
			names.push_back(L"Flounder Phil");
			names.push_back(L"Sad Clownfish");
			names.push_back(L"Calvin Caviar");
			names.push_back(L"Silly Seahorse");
			names.push_back(L"Cod & Chips");

			for(i=0;i<flowers.size();++i)
			{
				unavailable = false;
				for(j=0;j<fish.size();++j)
				{
					if(fish[j]->getDestFlower() == i)
					{
						unavailable = true;
						break;
					}
				}
				if(!unavailable)
					availableFlowers.push_back(i);
			}

			for(i=0;i<numPlayers;++i)
			{
				col.x = hue;
				col.y = 0.5f;
				col.z = 0.5f;
				col = ColourHelpers::hslToRgb(col);

				index = Random::getGlobal().getRandomInt(availableFlowers.size());
				index2 = Random::getGlobal().getRandomInt(names.size());
				fish.push_back(shared_ptr<Fish>(new Fish(flowers[availableFlowers[index]],
														 col,
														 names[i],
														 availableFlowers[index])));

				hue += 1.0f/(float)numPlayers;
				availableFlowers.erase(availableFlowers.begin()+index);
				names.erase(names.begin()+index2);
			}

			winner = L"";
			waitTime = true;
		}
	}*/
	else if(action == L"Action")
	{
  		for(i=0;i<fish.size();++i)
		{
			if(!fish[i]->getFishIsFalling())
				fish[i]->fall(false);
			else
			{
				availableFlowers.clear();
				for(int k=0;k<flowers.size();++k)
				{
					unavailable = false;
					for(j=0;j<fish.size();++j)
					{
						if(fish[j]->getDestFlower() == k)
						{
							unavailable = true;
							break;
						}
					}
					if(!unavailable)
						availableFlowers.push_back(k);
				}

				int index = Random::getGlobal().getRandomInt(availableFlowers.size());
				fish[i]->jumpTo(flowers[availableFlowers[index]], availableFlowers[index]);
				fish[i]->setNoFall(true);
			}
		}
	}
	else
	{
		for(i=0;i<fish.size();++i)
		{
			if(keyPressed == fish[i]->getDestKey())
			{
				/*if(fish[i]->getFishIsStatic())
					fish[i]->fall();
				else*/
				if(fish[i]->getFishIsFalling())
				{
					for(int k=0;k<flowers.size();++k)
					{
						unavailable = false;
						for(j=0;j<fish.size();++j)
						{
							if(fish[j]->getDestFlower() == k)
							{
								unavailable = true;
								break;
							}
						}
						if(!unavailable)
							availableFlowers.push_back(k);
					}

					int index = Random::getGlobal().getRandomInt(availableFlowers.size());
					fish[i]->jumpTo(flowers[availableFlowers[index]], availableFlowers[index]);
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlayState::keyUp(const std::wstring& action,
						  const std::wstring& keyPressed,
						  int keyModifier)
{
	int i;

	if(action == L"Action")
	{
		for(i=0;i<fish.size();++i)
		{
			fish[i]->setNoFall(false);

			if(fish[i]->getFishIsFalling() && (fish[i]->getDestFlower() == -1))
				fish[i]->updateDestKey();
		}
	}
}

//------------------------------------------------------------------------------
void GamePlayState::mouseMove(const TwoFloats& pos)
{
	
}
