//	GamePlayState.h - The main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef GAMEPLAYSTATE_H_
#define GAMEPLAYSTATE_H_

#include "drawing/TextLayout.h"
#include "ParticleGenerator.h"
#include "c0xHeaders.h"
#include "GameState.h"
#include "TwoFloats.h"
#include "Tweener.h"

#include <vector>
#include <string>

class Fish;

///	The main gameplay state.
class GamePlayState : public GameState
{
  public:
	///	Constructor.
	GamePlayState(Application& app, SounderPtr& sound);
	///	Destructor.
	~GamePlayState();

	///	Called frequently to update the state.
	void update(float delta);
	///	Called to draw the state.
	void draw(DrawerPtr& d);

	///	Called when a key down event is received.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);
	///	Called when a key up event is received.
	void keyUp(const std::wstring& action,
			   const std::wstring& keyPressed,
			   int keyModifier);

	///	Used to test connected line drawing.
	void mouseMove(const TwoFloats& pos);

	///	Returns the name of this state.
	const std::wstring getName() const {return L"GamePlayState";};
	///	Used to construct instances of this state.
	static GameStateCreator<GamePlayState> creator;
  private:
	///	The fish in the meadow.
	std::vector<shared_ptr<Fish> > fish;

	///	The flower positions in the meadow.
	std::vector<TwoFloats> flowers;

	///	The winning fish.
	std::wstring winner;
	///	Used to wait a wee while before the players can return to the title screen.
	float winWait;
	///	The winner text TextLayout.
	TextLayout winnerLayout;
	///	The winner text TextLayout.
	TextLayout winnerLayout2;
	///	The winner's particles.
	ParticleGenerator winParticles;

	///	Used to give players some time to pick their fish.
	bool waitTime;

	///	The title screen fade out.
	float fadeOut;
	///	The title screen fade out tweener.
	STweener<float> fadeOutTween;

	///	Our sounder.
	SounderPtr sounder;
};

#endif
