//	ParticleGenerator.cpp - Simple particle generator.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ParticleGenerator.h"
#include "drawing/Drawer.h"
#include "ColourHelpers.h"
#include "MiscHelper.h"
#include "Random.h"

#include <cmath>

//------------------------------------------------------------------------------
ParticleGenerator::ParticleGenerator():
minSpeed(2.0f),
maxSpeed(8.0f),
triggerParticles(false),
triggerCount(0.0f)
{

}

//------------------------------------------------------------------------------
ParticleGenerator::~ParticleGenerator()
{

}

//------------------------------------------------------------------------------
void ParticleGenerator::update(float delta)
{
	int i;
	float dir;
	ThreeFloats col;

	if(triggerParticles)
	{
		triggerCount -= delta;
		if(triggerCount < 0.0f)
		{
			col.x = Random::getGlobal().getRandomFloat();
			col.y = 0.5f;
			col.z = 0.75f;

			particles.push_back(Particle());
			particles.back().position = triggerPos;
			dir = Random::getGlobal().getRandomFloat() * 2.0f * NIALL_PI;
			particles.back().colour = ColourHelpers::hslToRgb(col);
			particles.back().direction.x = sinf(dir);
			particles.back().direction.y = cosf(dir);
			particles.back().rotation = Random::getGlobal().getRandomFloat() * 2.0f * NIALL_PI;
			particles.back().rotDir = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : -1.0f;
			particles.back().speed = minSpeed + (Random::getGlobal().getRandomFloat() * (maxSpeed-minSpeed));
			particles.back().life = 1.0f;

			triggerCount = 1.0f;
		}
	}

	for(i=(particles.size()-1);i>=0;--i)
	{
		particles[i].position += particles[i].direction * particles[i].speed * delta;
		particles[i].direction.y += 0.025f;
		particles[i].direction = particles[i].direction.normal();
		particles[i].rotation += particles[i].rotDir * 0.05f;
		particles[i].life -= (particles[i].speed/(maxSpeed*maxSpeed)) * 0.05f;
		if(particles[i].life <= 0.0f)
			particles.erase(particles.begin()+i);
	}
}

//------------------------------------------------------------------------------
void ParticleGenerator::draw(DrawerPtr d)
{
	unsigned int i;
	const TwoFloats size(1.0f);
	const TwoFloats& drawSize = d->getDrawingAreaSize();

	for(i=0;i<particles.size();++i)
	{
		d->drawRotImage(L"Fish",
						particles[i].position-(31.0f*size),
						particles[i].rotation,
						31.0f,
						size * particles[i].life,
						particles[i].colour,
						particles[i].life);
	}
}

//------------------------------------------------------------------------------
void ParticleGenerator::trigger(const TwoFloats& pos)
{
	/*int i;
	float dir;
	ThreeFloats col;
	const int NumTriggered = 8;

	for(i=0;i<NumTriggered;++i)
	{
		col.x = Random::getGlobal().getRandomFloat();
		col.y = 0.5f;
		col.z = 0.75f;

		particles.push_back(Particle());
		particles.back().position = pos;
		dir = Random::getGlobal().getRandomFloat() * 2.0f * NIALL_PI;
		particles.back().colour = ColourHelpers::hslToRgb(col);
		particles.back().direction.x = sinf(dir);
		particles.back().direction.y = cosf(dir);
		particles.back().rotation = Random::getGlobal().getRandomFloat() * 2.0f * NIALL_PI;
		particles.back().rotDir = (Random::getGlobal().getRandomFloat() > 0.5f) ? 1.0f : -1.0f;
		particles.back().speed = minSpeed + (Random::getGlobal().getRandomFloat() * (maxSpeed-minSpeed));
		particles.back().life = 1.0f;
	}*/
	triggerParticles = true;
	triggerPos = pos;
}
