//	TitleScreen.cpp - The title screen state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "FishGlobalData.h"
#include "TitleScreen.h"

//------------------------------------------------------------------------------
GameStateCreator<TitleScreen> TitleScreen::creator;

//------------------------------------------------------------------------------
TitleScreen::TitleScreen(Application& app, SounderPtr& sound):
GameState(app, sound),
fadeIn(1.0f),
fadeInTween(fadeIn, 0.0f, 0.02f)
{

}

//------------------------------------------------------------------------------
TitleScreen::~TitleScreen()
{

}

//------------------------------------------------------------------------------
void TitleScreen::update(float delta)
{
	if(!fadeInTween.getFinished())
		fadeInTween.tick(delta);
}

//------------------------------------------------------------------------------
void TitleScreen::draw(DrawerPtr& d)
{
	const TwoFloats centre(d->getDrawingAreaSize() * 0.5f);

	d->drawImage(L"Background", 0.0f);
	d->drawImage(L"Title", TwoFloats(centre.x-362.0f, 64.0f));

	d->drawText(L"Select number of players (1-9)",
				L"Font1",
				selectLayout,
				TwoFloats(0.0f, 690.0f),
				d->getDrawingAreaSize().x,
				TextLayout::Centre);

	if(!fadeInTween.getFinished())
	{
		d->fillRect(0.0f,
					d->getDrawingAreaSize(),
					ThreeFloats(141.0f/255.0f,
								220.0f/255.0f,
								127.0f/255.0f),
					fadeIn);
	}
}

//------------------------------------------------------------------------------
void TitleScreen::keyDown(const std::wstring& action,
						  const std::wstring& keyPressed,
						  int keyModifier)
{
	shared_ptr<FishGlobalData>& globalData = dynamic_pointer_cast<FishGlobalData>(getGlobalData());

	if(action == L"Escape")
		quit();
	else if(keyPressed == L"1")
	{
		globalData->setNumPlayers(1);
	}
	else if(keyPressed == L"2")
	{
		globalData->setNumPlayers(2);
		nextState(L"GamePlayState");
	}
	else if(keyPressed == L"3")
	{
		globalData->setNumPlayers(3);
		nextState(L"GamePlayState");
	}
	else if(keyPressed == L"4")
	{
		globalData->setNumPlayers(4);
		nextState(L"GamePlayState");
	}
	else if(keyPressed == L"5")
	{
		globalData->setNumPlayers(5);
		nextState(L"GamePlayState");
	}
	else if(keyPressed == L"6")
	{
		globalData->setNumPlayers(6);
		nextState(L"GamePlayState");
	}
	else if(keyPressed == L"7")
	{
		globalData->setNumPlayers(7);
		nextState(L"GamePlayState");
	}
	else if(keyPressed == L"8")
	{
		globalData->setNumPlayers(8);
		nextState(L"GamePlayState");
	}
	else if(keyPressed == L"9")
	{
		globalData->setNumPlayers(9);
		nextState(L"GamePlayState");
	}
}
